#ifndef __MD_LIB_OPTIMIZER_H__
#define __MD_LIB_OPTIMIZER_H__

#include <vector>
#include <MD/MD_FieldMap.h>
#include <MD/private/prefix.h>

namespace MD {

// A special-purpose hash table (of sorts) from field ids to offsets
// within a binary buffer.
class Optimizer {
public:
  MD_LIB_API Optimizer();
  MD_LIB_API Optimizer& operator=(const Optimizer& source);

  // Add a field to the optimizer and check its id for validity.
  MD_LIB_API bool set (MD::FIELDS::FieldID id, size_t offset);

  // Add a field to the optimizer and check its type and id for validity.
  MD_LIB_API bool set (MD::FIELDS::FieldID id, 
                       MD::DATATYPES::TypeID type, size_t offset);

  // Return the offset of a field, given the field ID, or 0 if the
  // field is not present or is invalid.
  MD_LIB_API size_t get (MD::FIELDS::FieldID id) const;

  // erases a field by id if it exists
  MD_LIB_API void erase (MD::FIELDS::FieldID id);

  // clears the optimizer
  MD_LIB_API void clear();

  MD_LIB_API size_t get_field_count() const { return m_field_count; }

private:
  const MD::MD_FieldMap & fmap;
  std::vector <size_t> m_vOffsets; // where is the field, in the buffer?
  std::vector <MD::FIELDS::FieldID> m_seen; // used for quick clears
  size_t m_field_count;
};

} // namespace

#endif // __MD_LIB_OPTIMIZER_H__
